# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: serve.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import common_pb2 as common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0bserve.proto\x12\x06\x64\x65ploy\x1a\x1bgoogle/protobuf/empty.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x0c\x63ommon.proto\"\x96\x03\n\x07Serving\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x1d\n\x06models\x18\x03 \x03(\x0b\x32\r.deploy.Model\x12-\n\x13\x64\x65ployment_endpoint\x18\x04 \x01(\x0b\x32\x10.deploy.Endpoint\x12)\n\x0fmodel_endpoints\x18\x05 \x03(\x0b\x32\x10.deploy.Endpoint\x12\"\n\x08resource\x18\x06 \x01(\x0b\x32\x10.common.Resource\x12-\n\x10\x63pu_scale_config\x18\x07 \x01(\x0b\x32\x13.deploy.ScaleConfig\x12\x13\n\x0b\x64\x65scription\x18\x08 \x01(\t\x12/\n\x08metadata\x18\t \x03(\x0b\x32\x1d.deploy.Serving.MetadataEntry\x12.\n\ncreated_at\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"7\n\x08\x45ndpoint\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\x12\x12\n\naccess_key\x18\x03 \x01(\t\"\xd6\x01\n\rServingStatus\x12\x33\n\rupload_status\x18\x01 \x01(\x0e\x32\x1c.deploy.ServingStatus.Status\x12\x33\n\rdeploy_status\x18\x02 \x01(\x0e\x32\x1c.deploy.ServingStatus.Status\"[\n\x06Status\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\x12\n\x0eSTATUS_PENDING\x10\x01\x12\x12\n\x0eSTATUS_SUCCESS\x10\x02\x12\x11\n\rSTATUS_FAILED\x10\x03\"L\n\x0bScaleConfig\x12\x14\n\x0cmin_replicas\x18\x01 \x01(\x05\x12\x14\n\x0cmax_replicas\x18\x02 \x01(\x05\x12\x11\n\tthreshold\x18\x03 \x01(\x02\"Z\n\x0eServingRequest\x12&\n\tauth_info\x18\x01 \x01(\x0b\x32\x13.common.NBXAuthInfo\x12 \n\x07serving\x18\x02 \x01(\x0b\x32\x0f.deploy.Serving\"\x91\x01\n\x14UpdateServingRequest\x12&\n\tauth_info\x18\x01 \x01(\x0b\x32\x13.common.NBXAuthInfo\x12 \n\x07serving\x18\x02 \x01(\x0b\x32\x0f.deploy.Serving\x12/\n\x0bupdate_mask\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.FieldMask\"[\n\x12ServingListRequest\x12&\n\tauth_info\x18\x01 \x01(\x0b\x32\x13.common.NBXAuthInfo\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\x0e\n\x06offset\x18\x03 \x01(\x05\"K\n\x13ServingListResponse\x12!\n\x08servings\x18\x01 \x03(\x0b\x32\x0f.deploy.Serving\x12\x11\n\tnext_page\x18\x02 \x01(\x05\"\xcc\x04\n\x05Model\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x18\n\x10serving_group_id\x18\x03 \x01(\t\x12.\n\ncreated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x04type\x18\x05 \x01(\x0e\x32\x19.deploy.Model.ServingType\x12\"\n\x08resource\x18\x06 \x01(\x0b\x32\x10.common.Resource\x12\x1a\n\x04\x63ode\x18\x07 \x01(\x0b\x32\x0c.common.Code\x12\"\n\x08\x65ndpoint\x18\x08 \x01(\x0b\x32\x10.deploy.Endpoint\x12+\n\npin_status\x18\t \x01(\x0e\x32\x17.deploy.Model.PinStatus\x12-\n\x08metadata\x18\n \x03(\x0b\x32\x1b.deploy.Model.MetadataEntry\x12%\n\x06status\x18\x0e \x01(\x0b\x32\x15.deploy.ServingStatus\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"E\n\x0bServingType\x12\x1c\n\x18SERVING_TYPE_UNSPECIFIED\x10\x00\x12\x18\n\x14SERVING_TYPE_NBOX_OP\x10\x01\"W\n\tPinStatus\x12\x1a\n\x16PIN_STATUS_UNSPECIFIED\x10\x00\x12\x15\n\x11PIN_STATUS_PINNED\x10\x01\x12\x17\n\x13PIN_STATUS_UNPINNED\x10\x02\"T\n\x0cModelRequest\x12&\n\tauth_info\x18\x01 \x01(\x0b\x32\x13.common.NBXAuthInfo\x12\x1c\n\x05model\x18\x02 \x01(\x0b\x32\r.deploy.Model\"m\n\x10ModelListRequest\x12&\n\tauth_info\x18\x01 \x01(\x0b\x32\x13.common.NBXAuthInfo\x12\x12\n\nserving_id\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x0e\n\x06offset\x18\x04 \x01(\x05\"E\n\x11ModelListResponse\x12\x1d\n\x06models\x18\x01 \x03(\x0b\x32\r.deploy.Model\x12\x11\n\tnext_page\x18\x02 \x01(\x05\"$\n\x08ModelLog\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04logs\x18\x02 \x03(\t2\xc4\x02\n\x0eServingService\x12\x31\n\x06\x43reate\x12\x16.deploy.ServingRequest\x1a\x0f.deploy.Serving\x12\x35\n\nGetServing\x12\x16.deploy.ServingRequest\x1a\x0f.deploy.Serving\x12G\n\x0cListServings\x12\x1a.deploy.ServingListRequest\x1a\x1b.deploy.ServingListResponse\x12?\n\rDeleteServing\x12\x16.deploy.ServingRequest\x1a\x16.google.protobuf.Empty\x12>\n\rUpdateServing\x12\x1c.deploy.UpdateServingRequest\x1a\x0f.deploy.Serving2\x9d\x03\n\x0cModelService\x12\x36\n\x05Watch\x12\x14.deploy.ModelRequest\x1a\x15.deploy.ServingStatus0\x01\x12\x32\n\x0bUploadModel\x12\x14.deploy.ModelRequest\x1a\r.deploy.Model\x12-\n\x06\x44\x65ploy\x12\x14.deploy.ModelRequest\x1a\r.deploy.Model\x12\x35\n\tModelLogs\x12\x14.deploy.ModelRequest\x1a\x10.deploy.ModelLog0\x01\x12\x41\n\nListModels\x12\x18.deploy.ModelListRequest\x1a\x19.deploy.ModelListResponse\x12;\n\x0b\x44\x65leteModel\x12\x14.deploy.ModelRequest\x1a\x16.google.protobuf.Empty\x12;\n\x0bSetModelPin\x12\x14.deploy.ModelRequest\x1a\x16.google.protobuf.EmptyB4Z2github.com/NimbleBoxAI/jobs-architecture/deploy_pbb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'serve_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2github.com/NimbleBoxAI/jobs-architecture/deploy_pb'
  _SERVING_METADATAENTRY._options = None
  _SERVING_METADATAENTRY._serialized_options = b'8\001'
  _MODEL_METADATAENTRY._options = None
  _MODEL_METADATAENTRY._serialized_options = b'8\001'
  _SERVING._serialized_start=134
  _SERVING._serialized_end=540
  _SERVING_METADATAENTRY._serialized_start=493
  _SERVING_METADATAENTRY._serialized_end=540
  _ENDPOINT._serialized_start=542
  _ENDPOINT._serialized_end=597
  _SERVINGSTATUS._serialized_start=600
  _SERVINGSTATUS._serialized_end=814
  _SERVINGSTATUS_STATUS._serialized_start=723
  _SERVINGSTATUS_STATUS._serialized_end=814
  _SCALECONFIG._serialized_start=816
  _SCALECONFIG._serialized_end=892
  _SERVINGREQUEST._serialized_start=894
  _SERVINGREQUEST._serialized_end=984
  _UPDATESERVINGREQUEST._serialized_start=987
  _UPDATESERVINGREQUEST._serialized_end=1132
  _SERVINGLISTREQUEST._serialized_start=1134
  _SERVINGLISTREQUEST._serialized_end=1225
  _SERVINGLISTRESPONSE._serialized_start=1227
  _SERVINGLISTRESPONSE._serialized_end=1302
  _MODEL._serialized_start=1305
  _MODEL._serialized_end=1893
  _MODEL_METADATAENTRY._serialized_start=493
  _MODEL_METADATAENTRY._serialized_end=540
  _MODEL_SERVINGTYPE._serialized_start=1735
  _MODEL_SERVINGTYPE._serialized_end=1804
  _MODEL_PINSTATUS._serialized_start=1806
  _MODEL_PINSTATUS._serialized_end=1893
  _MODELREQUEST._serialized_start=1895
  _MODELREQUEST._serialized_end=1979
  _MODELLISTREQUEST._serialized_start=1981
  _MODELLISTREQUEST._serialized_end=2090
  _MODELLISTRESPONSE._serialized_start=2092
  _MODELLISTRESPONSE._serialized_end=2161
  _MODELLOG._serialized_start=2163
  _MODELLOG._serialized_end=2199
  _SERVINGSERVICE._serialized_start=2202
  _SERVINGSERVICE._serialized_end=2526
  _MODELSERVICE._serialized_start=2529
  _MODELSERVICE._serialized_end=2942
# @@protoc_insertion_point(module_scope)
