# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/relics.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12proto/relics.proto\"\xa7\x01\n\x05Relic\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\ncreated_on\x18\x03 \x01(\x03\x12\x15\n\rlast_modified\x18\x04 \x01(\x03\x12\x0f\n\x07starred\x18\x05 \x01(\x08\x12\x0c\n\x04tags\x18\x06 \x03(\t\x12\x14\n\x0cworkspace_id\x18\x07 \x01(\t\x12$\n\x0b\x62ucket_meta\x18\x64 \x01(\x0b\x32\x0f.BucketMetadata\"\xa2\x02\n\x0e\x42ucketMetadata\x12\x13\n\x0b\x62ucket_name\x18\x01 \x01(\t\x12\x0e\n\x06region\x18\x02 \x01(\t\x12(\n\x07\x62\x61\x63kend\x18\x03 \x01(\x0e\x32\x17.BucketMetadata.Backend\x12\x34\n\x0b\x62ucket_tags\x18\x04 \x03(\x0b\x32\x1f.BucketMetadata.BucketTagsEntry\x1a\x31\n\x0f\x42ucketTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"X\n\x07\x42\x61\x63kend\x12\t\n\x05UNSET\x10\x00\x12\n\n\x06\x41WS_S3\x10\x01\x12\x0b\n\x07GCP_GCS\x10\x02\x12\x0e\n\nAZURE_BLOB\x10\x03\x12\n\n\x06OCI_OB\x10\x04\x12\r\n\tDO_SPACES\x10\x05\"\xda\x03\n\tRelicFile\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\ncreated_on\x18\x02 \x01(\x03\x12\x15\n\rlast_modified\x18\x03 \x01(\x03\x12\x0f\n\x07starred\x18\x04 \x01(\x08\x12\x0c\n\x04tags\x18\x05 \x03(\t\x12\x0c\n\x04size\x18\x06 \x01(\x03\x12\x0f\n\x07\x63omment\x18\x07 \x01(\t\x12\x10\n\x08username\x18\x08 \x01(\t\x12\"\n\x04type\x18\t \x01(\x0e\x32\x14.RelicFile.RelicType\x12\x14\n\x0cworkspace_id\x18\n \x01(\t\x12\x12\n\nrelic_name\x18\x0b \x01(\t\x12\x10\n\x08relic_id\x18\x0c \x01(\t\x12\x0b\n\x03url\x18\x64 \x01(\t\x12(\n\x07headers\x18\x65 \x03(\x0b\x32\x17.RelicFile.HeadersEntry\x12\"\n\x04\x62ody\x18\x66 \x03(\x0b\x32\x14.RelicFile.BodyEntry\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a+\n\tBodyEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\",\n\tRelicType\x12\x08\n\x04\x46ILE\x10\x00\x12\n\n\x06\x46OLDER\x10\x01\x12\t\n\x05RELIC\x10\x02\x62\x06proto3')



_RELIC = DESCRIPTOR.message_types_by_name['Relic']
_BUCKETMETADATA = DESCRIPTOR.message_types_by_name['BucketMetadata']
_BUCKETMETADATA_BUCKETTAGSENTRY = _BUCKETMETADATA.nested_types_by_name['BucketTagsEntry']
_RELICFILE = DESCRIPTOR.message_types_by_name['RelicFile']
_RELICFILE_HEADERSENTRY = _RELICFILE.nested_types_by_name['HeadersEntry']
_RELICFILE_BODYENTRY = _RELICFILE.nested_types_by_name['BodyEntry']
_BUCKETMETADATA_BACKEND = _BUCKETMETADATA.enum_types_by_name['Backend']
_RELICFILE_RELICTYPE = _RELICFILE.enum_types_by_name['RelicType']
Relic = _reflection.GeneratedProtocolMessageType('Relic', (_message.Message,), {
  'DESCRIPTOR' : _RELIC,
  '__module__' : 'proto.relics_pb2'
  # @@protoc_insertion_point(class_scope:Relic)
  })
_sym_db.RegisterMessage(Relic)

BucketMetadata = _reflection.GeneratedProtocolMessageType('BucketMetadata', (_message.Message,), {

  'BucketTagsEntry' : _reflection.GeneratedProtocolMessageType('BucketTagsEntry', (_message.Message,), {
    'DESCRIPTOR' : _BUCKETMETADATA_BUCKETTAGSENTRY,
    '__module__' : 'proto.relics_pb2'
    # @@protoc_insertion_point(class_scope:BucketMetadata.BucketTagsEntry)
    })
  ,
  'DESCRIPTOR' : _BUCKETMETADATA,
  '__module__' : 'proto.relics_pb2'
  # @@protoc_insertion_point(class_scope:BucketMetadata)
  })
_sym_db.RegisterMessage(BucketMetadata)
_sym_db.RegisterMessage(BucketMetadata.BucketTagsEntry)

RelicFile = _reflection.GeneratedProtocolMessageType('RelicFile', (_message.Message,), {

  'HeadersEntry' : _reflection.GeneratedProtocolMessageType('HeadersEntry', (_message.Message,), {
    'DESCRIPTOR' : _RELICFILE_HEADERSENTRY,
    '__module__' : 'proto.relics_pb2'
    # @@protoc_insertion_point(class_scope:RelicFile.HeadersEntry)
    })
  ,

  'BodyEntry' : _reflection.GeneratedProtocolMessageType('BodyEntry', (_message.Message,), {
    'DESCRIPTOR' : _RELICFILE_BODYENTRY,
    '__module__' : 'proto.relics_pb2'
    # @@protoc_insertion_point(class_scope:RelicFile.BodyEntry)
    })
  ,
  'DESCRIPTOR' : _RELICFILE,
  '__module__' : 'proto.relics_pb2'
  # @@protoc_insertion_point(class_scope:RelicFile)
  })
_sym_db.RegisterMessage(RelicFile)
_sym_db.RegisterMessage(RelicFile.HeadersEntry)
_sym_db.RegisterMessage(RelicFile.BodyEntry)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _BUCKETMETADATA_BUCKETTAGSENTRY._options = None
  _BUCKETMETADATA_BUCKETTAGSENTRY._serialized_options = b'8\001'
  _RELICFILE_HEADERSENTRY._options = None
  _RELICFILE_HEADERSENTRY._serialized_options = b'8\001'
  _RELICFILE_BODYENTRY._options = None
  _RELICFILE_BODYENTRY._serialized_options = b'8\001'
  _RELIC._serialized_start=23
  _RELIC._serialized_end=190
  _BUCKETMETADATA._serialized_start=193
  _BUCKETMETADATA._serialized_end=483
  _BUCKETMETADATA_BUCKETTAGSENTRY._serialized_start=344
  _BUCKETMETADATA_BUCKETTAGSENTRY._serialized_end=393
  _BUCKETMETADATA_BACKEND._serialized_start=395
  _BUCKETMETADATA_BACKEND._serialized_end=483
  _RELICFILE._serialized_start=486
  _RELICFILE._serialized_end=960
  _RELICFILE_HEADERSENTRY._serialized_start=823
  _RELICFILE_HEADERSENTRY._serialized_end=869
  _RELICFILE_BODYENTRY._serialized_start=871
  _RELICFILE_BODYENTRY._serialized_end=914
  _RELICFILE_RELICTYPE._serialized_start=916
  _RELICFILE_RELICTYPE._serialized_end=960
# @@protoc_insertion_point(module_scope)
